﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

namespace NET4_CS
{
 class TPLDemo
 {

  /// <summary>
  ///  Parallel.For
  /// </summary>
  public static void RunFor()
  {
   Parallel.For(1, 10, i => { Console.WriteLine(System.Threading.Thread.CurrentThread.ManagedThreadId + ": " + i); });
  }

  /// <summary>
  ///  Parallel.ForEach
  /// </summary>
  public static void RunForEach()
  {
   List<int> Zahlen = new List<int>() { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
   Parallel.ForEach(Zahlen, z => { Console.WriteLine(System.Threading.Thread.CurrentThread.ManagedThreadId + ": " + z); });
  }

  /// <summary>
  ///  Parallel.Invoke
  /// </summary>
  public static void RunInvoke()
  {

   // müssen alle void sein!
   Parallel.Invoke(Worker1, Worker2, Worker3);
  }

  /// <summary>
  ///  Task.Factory.StartNew
  /// </summary>
  public static void RunFactory()
  {
   Task<int> t = Task.Factory.StartNew<int>(Worker4);
   while (!t.IsCompleted) { Console.Write("."); }
   Console.WriteLine("Result: " + t.Result);
  }

  public static int Worker4()
  {
   for (int i = 0; i < 10; i++)
   {
    Console.WriteLine("W1: " + System.Threading.Thread.CurrentThread.ManagedThreadId + ": " + i);
   }
   return 10;
  }

  static CancellationTokenSource cts;

  /// <summary>
  ///  Task.Factory.StartNew mit Abbruch
  /// </summary>
  public static void RunFactoryCancel()
  {
   cts = new CancellationTokenSource();

   // Starte mit Abbruchoption
   Task<int> t = Task.Factory.StartNew<int>(() => Worker5(cts.Token));
   int i = 0;
   // Wartet, bis fertig
   while (!t.IsCompleted)
   {
    i++; Console.Write(".");
    System.Threading.Thread.Sleep(2);
    if (i > 30) cts.Cancel(); // künstlicher Abbruch nach 30 Schleifendurchläufen
   }
   if (t.IsFaulted) { Console.WriteLine("Cancelt!"); }
   else { Console.WriteLine("Result: " + t.Result); }
  }


  public static int Worker5(CancellationToken ct)
  {
   int count = 0;
   for (int i = 0; i < 10; i++)
   {
    count = i;
    Console.WriteLine("Worker5 in Thread " + System.Threading.Thread.CurrentThread.ManagedThreadId + ": " + i + ". Schritt!");
    System.Threading.Thread.Sleep(10);
    if (ct.IsCancellationRequested) { Console.WriteLine("Worker5 in Thread " + System.Threading.Thread.CurrentThread.ManagedThreadId + ": Abruch!"); return 0; }
   }
   return count;
  }



  public static void Worker1()
  {
   for (int i = 0; i < 10; i++)
   {
    Console.WriteLine("W1: " + System.Threading.Thread.CurrentThread.ManagedThreadId + ": " + i);
   }

  }


  public static void Worker2()
  {
   for (int i = 0; i < 10; i++)
   {
    Console.WriteLine("W2: " + System.Threading.Thread.CurrentThread.ManagedThreadId + ": " + i);
   }
  }


  public static void Worker3()
  {
   for (int i = 0; i < 10; i++)
   {
    Console.WriteLine("W3: " + System.Threading.Thread.CurrentThread.ManagedThreadId + ": " + i);
   }
  }


  public static void LockDemo()
  {
   ReaderWriterLockSlim l = new ReaderWriterLockSlim();

   l.EnterReadLock();

   l.ExitReadLock();

  }
 }
}
